# -*- coding: utf-8 -*-
import requests
from docopt import Dict
from irc3.plugins.command import command
from irc3.utils import IrcString

from . import Plugin


class Coins(Plugin):
    API_URL = 'https://api.cryptowat.ch/markets/{market}/{crypto}{currency}/summary'
    CURRENCIES = {
        'usd': '$',
        'eur': '€',
        'eth': 'Ξ',
        'btc': '฿',
        'xmr': 'xmr',
    }

    @command
    def btc(self, mask: IrcString, target: IrcString, args: Dict):
        """Gets the Bitcoin values from cryptowatch.

        %%btc [<currency>]
        """
        return self.cryptowat_summary('btc', 'coinbase', args)
        
    @command
    def eth(self, mask: IrcString, target: IrcString, args: Dict):
        """Gets the Ethereum values from cryptowatch.

        %%eth [<currency>]
        """
        return self.cryptowat_summary('eth', 'coinbase', args)
        
    @command
    def xmr(self, mask: IrcString, target: IrcString, args: Dict):
        """Gets the Monreo values from cryptowatch.

        %%xmr [<currency>]
        """
        return self.cryptowat_summary('xmr', 'bitfinex', args)

    def cryptowat_summary(self, crypto: str, market: market, args: Dict):
        currency = args.get('<currency>', 'usd')

        if currency not in self.CURRENCIES or crypto == currency:
            return '\x02[{}]\x02 Can\'t convert or invalid currency: {}'.format(crypto.upper(), currency)

        data = requests.get(self.API_URL.format(crypto=crypto, market=market, currency=currency))
        if not data:
            return '\x02[{}]\x02 No data received'.format(crypto)

        result = data.json()['result']
        return '\x02[{crypto}]\x02 ' \
               'Current: \x02\x0307{currency}{last:,.2f}\x0F - ' \
               'High: \x02\x0303{currency}{high:,.2f}\x0F - ' \
               'Low: \x02\x0304{currency}{low:,.2f}\x0F - ' \
               'Change: \x02\x0307{change:,.2f}%\x0F - ' \
               'Volume: \x02\x0307{volume}\x0F' \
            .format(crypto=crypto.upper(),
                    currency=self.CURRENCIES[currency],
                    last=result['price']['last'],
                    high=result['price']['high'],
                    low=result['price']['low'],
                    change=result['price']['change']['percentage'] * 100,
                    volume=result['volume'])
